__name__ = 'pixel-artist'
__version__ = "0.0.2"
__author__ = "Alexandre Pinto"
__author_email__ = "alexandpinto@gmail.com"
__description__ = 'A command line utility to build pixelated images.'
__url__ = 'https://github.com/alexpnt/pixel-artist'
__docs__ = 'https://github.com/alexpnt/pixel-artist'
__issues__ = 'https://github.com/alexpnt/pixel-artist/issues'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022 Alexandre Pinto'
__all__ = ["__version__", "__author__", "__author_email__", "__url__", "__license__"]
